/*
 * HCSR04.cpp - Library for using the HCSR04 distance sensor.
 * Created by Tyler J. Sekosan, March 3, 2019.
 * Released into the public domain.
 */
 
#include "Arduino.h" // includes standard Arduino code
#include "HCSR04.h" // includes definitions from the header file

HCSR04ProxSensor::HCSR04ProxSensor(int echoPin, int triggerPin) // constructor for our sensor object
{
  _echoPin = echoPin; // stores the number of the pin used as the echo pin for each sensor instance we use !!!!!
  pinMode(echoPin, INPUT); // sets the echo pin as an input
  _triggerPin = triggerPin; // stores the number of the pin used as the trigger pin for each sensor instance we use !!!!!
  pinMode(triggerPin, OUTPUT); // sets the trigger pin as an output
}

float HCSR04ProxSensor::readSensor() // this is the function used to find the distance (in cm) an object is from the sensor
{
  _lastValue = _cmDistance; // this stores the previous distance before the function runs again. This last value is useful to determine if an object is moving towards or away from the sensor 
  digitalWrite(_triggerPin, LOW); // this sequence of digitalWrite lines are used to send the actual signal from the sensor via the trigger pin by turning the pin high for 10 microseconds
  delayMicroseconds(2);
  digitalWrite(_triggerPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(_triggerPin, LOW);
  _time = pulseIn(_echoPin, HIGH); // we store the time it takes for the sound wave emitted by the sensor to echo back
  _cmDistance = (_time/58); // this calculates the distance of an object (in cm) from the sensor
  return _cmDistance; // the return command sends the distance we calculated back to the main code that's using this library, which can be used to define variables, print on a screen, etc
}

float HCSR04ProxSensor::getLastValue() // since the readSensor function is being looped in the main code, this function returns the value of the distance before the next loop is run
{
  return _lastValue;
}
