// Code for the Lonely Boat Projec0000t by Tyler Sekosan. Developed as part of the Makecourse at the University of South Florida - Tampa. 

#include "Arduino.h" // includes standard Arduino code
#include "HCSR04.h"  // includes the proximity sensor library I created to make the code easier to read

int IN1 = 2;    // These "IN" pins are the pins on the L298N DC motor controller, essentially inputs for digital outputs from pins on the arduino
int IN2 = 3;    // IN1 and IN2 are for motor A and the ENA pins (described below)
int IN3 = 4;    // IN3 and IN4 are pins for motor B and the ENB pins
int IN4 = 7;

int ENA = 10; // These are the enable pins on the motor controller that basically tell the motor how fast to go via PWM signals from the arduino. ENA is for motor 
int ENB = 5; // Enable pin for motor B
int ENA2 = 11;
int ENB2 = 6;

int triggerPin = 13; // Output for the trigger that makes the sensor send out a pulse that bounces back to measure distance. 
int echoPin = 12; // Input pin on the arduino that receives the signal that returned by the pulse emitted by the trigger pin of the sensor. 
 

HCSR04ProxSensor sensor(echoPin,triggerPin); // This creates a proximity sensor object for the sensor we are using, part of the library I created

void setup() {

  Serial.begin(9600); // Begin serial communication, so I can monitor the distance measured from the sensor on the serial monitor
  
  pinMode(IN1, OUTPUT); // Here we are assigned the input pins on the L298N motor controller as OUTPUTS for the arduino, as they send signals to the controller
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  pinMode(ENA, OUTPUT);
  pinMode(ENB, OUTPUT);
  pinMode(ENA2, OUTPUT);
  pinMode(ENB2, OUTPUT);
  
  digitalWrite(IN1, HIGH); // Sets motor 1 spinning forward
  digitalWrite(IN2, LOW);
  
  digitalWrite(IN3, HIGH); // set motor 2 spinning forward. If I were to set IN4 high and IN3 low, the motor would switch directions
  digitalWrite(IN4, LOW);
 

}

void loop() {

  analogWrite(ENA, 255); // Here we are setting the motors to full speed (which corresponds with 255), so the boat will try to move forwards by default
  analogWrite(ENB, 255);
  analogWrite(ENA2, 255);
  analogWrite(ENB2, 255);

 float  distance = sensor.readSensor(); // This is the variable to hold the value for our distance reading from the sensor

 // In the if statement below, we are printing the distance read from the sensor to the serial monitor of the Arduino IDE. This is useful for testing and ensuring that the sensor is working as intended
 // Distance readings are in centimeters (cm)
  
  if(distance < 1000) { // if the distance measured is below 1000cm

    Serial.print("The distance is :  ");
  
    Serial.print(distance);
    Serial.println(" cm");

    delay(100);
  
  }

// In the if statement below, we are evaluating whether an object is close to the boat, but far enough to where we should begin turning away from it. In this case, we set one motor moving forwards 
// and the other motor is in reverse, causing the boat to turn

if(15 < distance < 60) {
   
   digitalWrite(IN1, LOW); // Set motor 1 reverse
   digitalWrite(IN2, HIGH);
 
}
  
// The next if statement evaluates if there is an object close to the boat, i.e within 15cm of the boat. If there is, the boat will begin to move backwards

  if(distance < 15){ 
     
  digitalWrite(IN1, LOW); // Set motor 1 reverse
  digitalWrite(IN2, HIGH);
  
  digitalWrite(IN3, LOW); // set motor 2 reverse
  digitalWrite(IN4, HIGH);

   delay(3000); // I was running into problems if I didn't have the next block of code, which sets the boat moving forward again after 3 seconds. This is to avoid an issue where the boat was trying to move forwards and back at the same time
     
  digitalWrite(IN1, HIGH); // Set motor 1 forwards
  digitalWrite(IN2, LOW);
  
  digitalWrite(IN3, HIGH); // set motor 2 forwards
  digitalWrite(IN4, LOW);

 
  }

  // This next if statement is evaluating if there is no object near the boat, meaning that there isn't an object within 100cm of the boat. In this case, the boat will begin to move forwards
  
  if(distance > 60) { 
      
  digitalWrite(IN1, HIGH); // Set motor 1 forward
  digitalWrite(IN2, LOW);
  
  digitalWrite(IN3, HIGH); // set motor 2 forward
  digitalWrite(IN4, LOW);
  
  }
  
  delay(200); // This is to give the boat time to figure out if it needs to change speed again on the next loop
  
}
